a {{
    color: #000;
}}
.msg-user {{
    color: #686868 !important;
    white-space: pre-wrap;
    width: 100%;
    max-width: 100%;
}}
.msg-bot {{
    white-space: pre-wrap;
    width: 100%;
    max-width: 100%;
}}
.cmd {{
    color: #4d4d4d;
}}
.ts {{
    color: #4d4d4d;
}}
.list {{
}}
pre {{
    color: #000;
    background-color: #e9e9e9;
    font-family: 'Lato';
    display: block;
}}
code {{
    color: #000;
}}